/*  $Header: /home/cvsroot/dvipdfmx/src/pdfdraw.h,v 1.2 2004/08/07 09:52:34 hirata Exp $
    
    This is dvipdfmx, an eXtended version of dvipdfm by Mark A. Wicks.

    Copyright (C) 2002 by Jin-Hwan Cho and Shunsaku Hirata,
    the dvipdfmx project team <dvipdfmx@project.ktug.or.kr>
    
    Copyright (C) 1998, 1999 by Mark A. Wicks <mwicks@kettering.edu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#ifndef _PDF_DRAW_H_
#define _PDF_DRAW_H_

#include "pdfdev.h"

typedef struct pdf_path pdf_path;

#define PDF_PATH_MOVETO   1
#define PDF_PATH_RMOVETO  2
#define PDF_PATH_LINETO   3
#define PDF_PATH_RLINETO  4
#define PDF_PATH_CURVETO  5
#define PDF_PATH_RCURVETO 6
#define PDF_PATH_CLOSE    7

#define pdf_copymatrix(m,n) do {\
   (m)->a = (n)->a; (m)->b = (n)->b;\
   (m)->c = (n)->c; (m)->d = (n)->d;\
   (m)->e = (n)->e; (m)->f = (n)->f;\
 } while (0)

#define pdf_setmatrix(m,p,q,r,s,t,u) do {\
   (m)->a = (p); (m)->b = (q);\
   (m)->c = (r); (m)->d = (s);\
   (m)->e = (t); (m)->f = (u);\
 } while (0)

/* m -> m x n */
#define pdf_mulmatrix(m,n) do {\
  double _tmp_a, _tmp_b, _tmp_c, _tmp_d; \
  _tmp_a = (m)->a; _tmp_b = (m)->b; \
  _tmp_c = (m)->c; _tmp_d = (m)->d; \
  (m)->a  = _tmp_a * ((n)->a) + _tmp_b * ((n)->c); \
  (m)->b  = _tmp_a * ((n)->b) + _tmp_b * ((n)->d); \
  (m)->c  = _tmp_c * ((n)->a) + _tmp_d * ((n)->c); \
  (m)->d  = _tmp_c * ((n)->b) + _tmp_d * ((n)->d); \
  (m)->e += _tmp_a * ((n)->e) + _tmp_c * ((n)->f); \
  (m)->f += _tmp_b * ((n)->e) + _tmp_d * ((n)->f); \
} while (0)

extern void pdf_invertmatrix (pdf_tmatrix *M);
extern void pdf_draw_dtransform_coord  (pdf_coord *p, const pdf_tmatrix *M);
extern void pdf_draw_idtransform_coord (pdf_coord *p, const pdf_tmatrix *M);
extern void pdf_draw_transform_coord   (pdf_coord *p, const pdf_tmatrix *M);
extern void pdf_draw_itransform_coord  (pdf_coord *p, const pdf_tmatrix *M);
extern void pdf_draw_transform_path    (pdf_path *p, const pdf_tmatrix *M);

extern void pdf_draw_setlinewidth  (double width);
extern void pdf_draw_setmiterlimit (double mlimit);
extern void pdf_draw_setlinecap    (int style);
extern void pdf_draw_setlinejoin   (int style);
extern void pdf_draw_setdash       (int count, double *pattern, double offset);

extern void pdf_draw_path    (const pdf_path *p, const char *p_op);

extern pdf_path *pdf_draw_new_path (void);
extern void pdf_draw_clear_path   (pdf_path *p);
extern void pdf_draw_release_path (pdf_path *p);
extern int  pdf_draw_copy_path    (pdf_path *dst, pdf_path *src);
extern int  pdf_draw_append_path  (pdf_path *p, pdf_coord *cp,
				   int type, double *pts);
extern int  pdf_draw_append_arc   (pdf_path *p, pdf_coord *cp,
				   double c_x , double c_y,
				   double r_x , double r_y,
				   double ang0, double ang1);

extern int  pdf_draw_append_arcn  (pdf_path *p, pdf_coord *cp,
				   double c_x , double c_y,
				   double r_x , double r_y,
				   double ang0, double ang1);

extern int  pdf_draw_append_bspline (pdf_path  *p, pdf_coord *cp,
				     double x0, double y0,
				     double x1, double y1,
				     double x2, double y2);


#endif /* _PDF_DRAW_H_ */
